@@ -31,6 +31,10 @@
 #include <stdio.h>
 
 #ifdef __APPLE__
+# include <TargetConditionals.h>
+#endif
+
+#if defined(__APPLE__) && !TARGET_OS_IPHONE
 # include <crt_externs.h>
 # define environ (*_NSGetEnviron())
 #else
@@ -225,8 +229,8 @@ int uv_spawn(uv_loop_t* loop, uv_process_t* process,
 
   if (pid == -1) {
 #if SPAWN_WAIT_EXEC
-    uv__close(signal_pipe[0]);
-    uv__close(signal_pipe[1]);
+    close(signal_pipe[0]);
+    close(signal_pipe[1]);
 #endif
     environ = save_our_env;
     goto error;
@@ -234,7 +238,7 @@ int uv_spawn(uv_loop_t* loop, uv_process_t* process,
 
   if (pid == 0) {
     if (stdin_pipe[0] >= 0) {
-      uv__close(stdin_pipe[1]);
+      close(stdin_pipe[1]);
       dup2(stdin_pipe[0],  STDIN_FILENO);
     } else {
       /* Reset flags that might be set by Node */
@@ -243,7 +247,7 @@ int uv_spawn(uv_loop_t* loop, uv_process_t* process,
     }
 
     if (stdout_pipe[1] >= 0) {
-      uv__close(stdout_pipe[0]);
+      close(stdout_pipe[0]);
       dup2(stdout_pipe[1], STDOUT_FILENO);
     } else {
       /* Reset flags that might be set by Node */
@@ -252,7 +256,7 @@ int uv_spawn(uv_loop_t* loop, uv_process_t* process,
     }
 
     if (stderr_pipe[1] >= 0) {
-      uv__close(stderr_pipe[0]);
+      close(stderr_pipe[0]);
       dup2(stderr_pipe[1], STDERR_FILENO);
     } else {
       /* Reset flags that might be set by Node */
@@ -280,7 +284,7 @@ int uv_spawn(uv_loop_t* loop, uv_process_t* process,
 
 #if SPAWN_WAIT_EXEC
   /* POLLHUP signals child has exited or execve()'d. */
-  uv__close(signal_pipe[1]);
+  close(signal_pipe[1]);
   do {
     pfd.fd = signal_pipe[0];
     pfd.events = POLLIN|POLLHUP;
@@ -290,7 +294,7 @@ int uv_spawn(uv_loop_t* loop, uv_process_t* process,
   while (status == -1 && (errno == EINTR || errno == ENOMEM));
 
   assert((status == 1) && "poll() on pipe read end failed");
-  uv__close(signal_pipe[0]);
+  close(signal_pipe[0]);
 #endif
 
   process->pid = pid;
@@ -302,7 +306,7 @@ int uv_spawn(uv_loop_t* loop, uv_process_t* process,
   if (stdin_pipe[1] >= 0) {
     assert(options.stdin_stream);
     assert(stdin_pipe[0] >= 0);
-    uv__close(stdin_pipe[0]);
+    close(stdin_pipe[0]);
     uv__nonblock(stdin_pipe[1], 1);
     flags = UV_WRITABLE | (options.stdin_stream->ipc ? UV_READABLE : 0);
     uv__stream_open((uv_stream_t*)options.stdin_stream, stdin_pipe[1],
@@ -312,7 +316,7 @@ int uv_spawn(uv_loop_t* loop, uv_process_t* process,
   if (stdout_pipe[0] >= 0) {
     assert(options.stdout_stream);
     assert(stdout_pipe[1] >= 0);
-    uv__close(stdout_pipe[1]);
+    close(stdout_pipe[1]);
     uv__nonblock(stdout_pipe[0], 1);
     flags = UV_READABLE | (options.stdout_stream->ipc ? UV_WRITABLE : 0);
     uv__stream_open((uv_stream_t*)options.stdout_stream, stdout_pipe[0],
@@ -322,7 +326,7 @@ int uv_spawn(uv_loop_t* loop, uv_process_t* process,
   if (stderr_pipe[0] >= 0) {
     assert(options.stderr_stream);
     assert(stderr_pipe[1] >= 0);
-    uv__close(stderr_pipe[1]);
+    close(stderr_pipe[1]);
     uv__nonblock(stderr_pipe[0], 1);
     flags = UV_READABLE | (options.stderr_stream->ipc ? UV_WRITABLE : 0);
     uv__stream_open((uv_stream_t*)options.stderr_stream, stderr_pipe[0],
@@ -333,12 +337,12 @@ int uv_spawn(uv_loop_t* loop, uv_process_t* process,
 
 error:
   uv__set_sys_error(process->loop, errno);
-  uv__close(stdin_pipe[0]);
-  uv__close(stdin_pipe[1]);
-  uv__close(stdout_pipe[0]);
-  uv__close(stdout_pipe[1]);
-  uv__close(stderr_pipe[0]);
-  uv__close(stderr_pipe[1]);
+  close(stdin_pipe[0]);
+  close(stdin_pipe[1]);
+  close(stdout_pipe[0]);
+  close(stdout_pipe[1]);
+  close(stderr_pipe[0]);
+  close(stderr_pipe[1]);
   return -1;
 }
 
