@@ -101,20 +101,23 @@ const char* uv_strerror(uv_err_t err) {
 #undef UV_STRERROR_GEN
 
 
-void uv__set_error(uv_loop_t* loop, uv_err_code code, int sys_error) {
+int uv__set_error(uv_loop_t* loop, uv_err_code code, int sys_error) {
   loop->last_err.code = code;
   loop->last_err.sys_errno_ = sys_error;
+  return -1;
 }
 
 
-void uv__set_sys_error(uv_loop_t* loop, int sys_error) {
+int uv__set_sys_error(uv_loop_t* loop, int sys_error) {
   loop->last_err.code = uv_translate_sys_error(sys_error);
   loop->last_err.sys_errno_ = sys_error;
+  return -1;
 }
 
 
-void uv__set_artificial_error(uv_loop_t* loop, uv_err_code code) {
+int uv__set_artificial_error(uv_loop_t* loop, uv_err_code code) {
   loop->last_err = uv__new_artificial_error(code);
+  return -1;
 }
 
 
@@ -177,57 +180,43 @@ int uv_ip6_name(struct sockaddr_in6* src, char* dst, size_t size) {
 }
 
 
-/* find matching ares handle in list */
+static int cmp_ares_tasks(const uv_ares_task_t* a, const uv_ares_task_t* b) {
+  if (a->sock < b->sock) return -1;
+  if (a->sock > b->sock) return 1;
+  return 0;
+}
+
+
+RB_GENERATE_STATIC(uv__ares_tasks, uv_ares_task_s, node, cmp_ares_tasks)
+
+
+/* add ares handle to list */
 void uv_add_ares_handle(uv_loop_t* loop, uv_ares_task_t* handle) {
-  handle->loop = loop;
-  handle->ares_next = loop->uv_ares_handles_;
-  handle->ares_prev = NULL;
-
-  if (loop->uv_ares_handles_) {
-    loop->uv_ares_handles_->ares_prev = handle;
-  }
-
-  loop->uv_ares_handles_ = handle;
+  assert(loop == handle->loop);
+  RB_INSERT(uv__ares_tasks, &loop->uv_ares_handles_, handle);
 }
 
+
 /* find matching ares handle in list */
-/* TODO: faster lookup */
 uv_ares_task_t* uv_find_ares_handle(uv_loop_t* loop, ares_socket_t sock) {
-  uv_ares_task_t* handle = loop->uv_ares_handles_;
-
-  while (handle != NULL) {
-    if (handle->sock == sock) {
-      break;
-    }
-    handle = handle->ares_next;
-  }
-
-  return handle;
+  uv_ares_task_t handle;
+  handle.sock = sock;
+  return RB_FIND(uv__ares_tasks, &loop->uv_ares_handles_, &handle);
 }
 
+
 /* remove ares handle in list */
 void uv_remove_ares_handle(uv_ares_task_t* handle) {
-  uv_loop_t* loop = handle->loop;
-
-  if (handle == loop->uv_ares_handles_) {
-    loop->uv_ares_handles_ = handle->ares_next;
-  }
-
-  if (handle->ares_next) {
-    handle->ares_next->ares_prev = handle->ares_prev;
-  }
-
-  if (handle->ares_prev) {
-    handle->ares_prev->ares_next = handle->ares_next;
-  }
+  RB_REMOVE(uv__ares_tasks, &handle->loop->uv_ares_handles_, handle);
 }
 
 
 /* Returns 1 if the uv_ares_handles_ list is empty. 0 otherwise. */
 int uv_ares_handles_empty(uv_loop_t* loop) {
-  return loop->uv_ares_handles_ ? 0 : 1;
+  return RB_EMPTY(&loop->uv_ares_handles_);
 }
 
+
 int uv_tcp_bind(uv_tcp_t* handle, struct sockaddr_in addr) {
   if (handle->type != UV_TCP || addr.sin_family != AF_INET) {
     uv__set_artificial_error(handle->loop, UV_EFAULT);
@@ -237,6 +226,7 @@ int uv_tcp_bind(uv_tcp_t* handle, struct sockaddr_in addr) {
   return uv__tcp_bind(handle, addr);
 }
 
+
 int uv_tcp_bind6(uv_tcp_t* handle, struct sockaddr_in6 addr) {
   if (handle->type != UV_TCP || addr.sin6_family != AF_INET6) {
     uv__set_artificial_error(handle->loop, UV_EFAULT);
@@ -246,6 +236,7 @@ int uv_tcp_bind6(uv_tcp_t* handle, struct sockaddr_in6 addr) {
   return uv__tcp_bind6(handle, addr);
 }
 
+
 int uv_udp_bind(uv_udp_t* handle, struct sockaddr_in addr,
     unsigned int flags) {
   if (handle->type != UV_UDP || addr.sin_family != AF_INET) {
@@ -256,6 +247,7 @@ int uv_udp_bind(uv_udp_t* handle, struct sockaddr_in addr,
   return uv__udp_bind(handle, addr, flags);
 }
 
+
 int uv_udp_bind6(uv_udp_t* handle, struct sockaddr_in6 addr,
     unsigned int flags) {
   if (handle->type != UV_UDP || addr.sin6_family != AF_INET6) {
@@ -266,6 +258,7 @@ int uv_udp_bind6(uv_udp_t* handle, struct sockaddr_in6 addr,
   return uv__udp_bind6(handle, addr, flags);
 }
 
+
 int uv_tcp_connect(uv_connect_t* req,
                    uv_tcp_t* handle,
                    struct sockaddr_in address,
@@ -278,6 +271,7 @@ int uv_tcp_connect(uv_connect_t* req,
   return uv__tcp_connect(req, handle, address, cb);
 }
 
+
 int uv_tcp_connect6(uv_connect_t* req,
                     uv_tcp_t* handle,
                     struct sockaddr_in6 address,
