@@ -86,7 +86,7 @@ static void ipc_on_connection(uv_stream_t* server, int status) {
   uv_tcp_t* conn;
 
   if (!connection_accepted) {
-    /* 
+    /*
      * Accept the connection and close it.  Also let the other
      * side know.
      */
@@ -121,7 +121,7 @@ static int ipc_helper(int listen_after_write) {
    * data is transfered over the channel. XXX edit this comment after handle
    * transfer is added.
    */
-  
+
   uv_write_t write_req;
   int r;
   uv_buf_t buf;
@@ -131,6 +131,9 @@ static int ipc_helper(int listen_after_write) {
 
   uv_pipe_open(&channel, 0);
 
+  ASSERT(uv_is_readable((uv_stream_t*) &channel));
+  ASSERT(uv_is_writable((uv_stream_t*) &channel));
+
   r = uv_tcp_init(uv_default_loop(), &tcp_server);
   ASSERT(r == 0);
 
@@ -205,7 +208,7 @@ static int stdio_over_pipes_helper() {
   uv_buf_t buf[ARRAY_SIZE(buffers)];
   int r, i;
   uv_loop_t* loop = uv_default_loop();
-  
+
   ASSERT(UV_NAMED_PIPE == uv_guess_handle(0));
   ASSERT(UV_NAMED_PIPE == uv_guess_handle(1));
 
@@ -268,6 +271,11 @@ static int maybe_run_test(int argc, char **argv) {
     return ipc_helper(1);
   }
 
+  if (strcmp(argv[1], "ipc_send_recv_helper") == 0) {
+    int ipc_send_recv_helper(void); /* See test-ipc-send-recv.c */
+    return ipc_send_recv_helper();
+  }
+
   if (strcmp(argv[1], "stdio_over_pipes_helper") == 0) {
     return stdio_over_pipes_helper();
   }
