@@ -50,8 +50,8 @@ class libuv_build_ext(build_ext):
     libuv_dir      = os.path.join('deps', 'libuv')
     libuv_repo     = 'https://github.com/joyent/libuv.git'
     libuv_branch   = 'master'
-    libuv_revision = '3de0411'
-    libuv_patches  = []
+    libuv_revision = '31ff986'
+    libuv_patches  = ['patches/c-ares_naptr_support.patch']
 
     user_options = build_ext.user_options
     user_options.extend([
@@ -67,8 +67,10 @@ class libuv_build_ext(build_ext):
         self.libuv_force_fetch = 0
 
     def build_extensions(self):
-        if self.libuv_force_fetch or self.libuv_clean_compile:
-            self.force = 1
+        if self.compiler.compiler_type == 'mingw32':
+            # Dirty hack to avoid linking with more than one C runtime when using MinGW
+            self.compiler.dll_libraries = [lib for lib in self.compiler.dll_libraries if not lib.startswith('msvcr')]
+        self.force = self.libuv_force_fetch or self.libuv_clean_compile
         self.get_libuv()
         build_ext.build_extensions(self)
 
@@ -92,7 +94,7 @@ class libuv_build_ext(build_ext):
         def download_libuv():
             log.info('Downloading libuv...')
             makedirs(self.libuv_dir)
-            exec_process(['git', 'clone', '--depth', '1', '-b', self.libuv_branch, self.libuv_repo, self.libuv_dir])
+            exec_process(['git', 'clone', '-b', self.libuv_branch, self.libuv_repo, self.libuv_dir])
             exec_process(['git', 'checkout', self.libuv_revision], cwd=self.libuv_dir)
         def patch_libuv():
             log.info('Patching libuv...')
