@@ -19,7 +19,7 @@ host_cb(void *arg, int status, int timeouts, struct hostent *hostent)
     char **ptr;
     ares_cb_data_t *data;
     DNSResolver *self;
-    PyObject *callback, *dns_name, *dns_status, *dns_timeouts, *dns_aliases, *dns_result, *tmp, *result;
+    PyObject *callback, *dns_name, *errorno, *dns_aliases, *dns_addrlist, *dns_result, *tmp, *result;
 
     ASSERT(arg);
 
@@ -31,58 +31,71 @@ host_cb(void *arg, int status, int timeouts, struct hostent *hostent)
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    if (hostent != NULL) {
-        dns_name = PyString_FromString(hostent->h_name);
-    } else {
+    if (status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)status);
+        dns_result = Py_None;
         Py_INCREF(Py_None);
-        dns_name = Py_None;
+        goto callback;
     }
 
-    dns_status = PyLong_FromLong((long) status);
-    dns_timeouts = PyLong_FromLong((long) timeouts);
     dns_aliases = PyList_New(0);
-    dns_result = PyList_New(0);
+    dns_addrlist = PyList_New(0);
+    dns_result = PyTuple_New(3);
 
-    if (!(dns_status && dns_timeouts && dns_name && dns_aliases && dns_result)) {
+    if (!(dns_aliases && dns_addrlist && dns_result)) {
         PyErr_NoMemory();
-        PyErr_WriteUnraisable(callback);
-        goto host_end;
+        PyErr_WriteUnraisable(Py_None);
+        Py_XDECREF(dns_aliases);
+        Py_XDECREF(dns_addrlist);
+        Py_XDECREF(dns_result);
+        errorno = PyInt_FromLong((long)ARES_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
     }
 
-    if (hostent != NULL) {
-        for (ptr = hostent->h_aliases; *ptr != NULL; ptr++) {
-            if (*ptr != hostent->h_name && strcmp(*ptr, hostent->h_name)) {
-                tmp = PyString_FromString(*ptr);
-                if (tmp == NULL) {
-                    break;
-                }
-                PyList_Append(dns_aliases, tmp);
-                Py_DECREF(tmp);
-            }
-        }
-        for (ptr = hostent->h_addr_list; *ptr != NULL; ptr++) {
-            if (hostent->h_addrtype == AF_INET) {
-                uv_inet_ntop(AF_INET, *ptr, ip, INET_ADDRSTRLEN);
-                tmp = PyString_FromString(ip);
-            } else {
-                uv_inet_ntop(AF_INET6, *ptr, ip, INET6_ADDRSTRLEN);
-                tmp = PyString_FromString(ip);
-            }
+    for (ptr = hostent->h_aliases; *ptr != NULL; ptr++) {
+        if (*ptr != hostent->h_name && strcmp(*ptr, hostent->h_name)) {
+            tmp = PYUVString_FromString(*ptr);
             if (tmp == NULL) {
                 break;
             }
-            PyList_Append(dns_result, tmp);
+            PyList_Append(dns_aliases, tmp);
             Py_DECREF(tmp);
         }
     }
+    for (ptr = hostent->h_addr_list; *ptr != NULL; ptr++) {
+        if (hostent->h_addrtype == AF_INET) {
+            uv_inet_ntop(AF_INET, *ptr, ip, INET_ADDRSTRLEN);
+            tmp = PYUVString_FromString(ip);
+        } else if (hostent->h_addrtype == AF_INET6) {
+            uv_inet_ntop(AF_INET6, *ptr, ip, INET6_ADDRSTRLEN);
+            tmp = PYUVString_FromString(ip);
+        } else {
+            continue;
+        }
+        if (tmp == NULL) {
+            break;
+        }
+        PyList_Append(dns_addrlist, tmp);
+        Py_DECREF(tmp);
+    }
+    dns_name = PYUVString_FromString(hostent->h_name);
 
-    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_timeouts, dns_name, dns_aliases, dns_result, NULL);
+    PyTuple_SET_ITEM(dns_result, 0, dns_name);
+    PyTuple_SET_ITEM(dns_result, 1, dns_aliases);
+    PyTuple_SET_ITEM(dns_result, 2, dns_addrlist);
+    errorno = Py_None;
+    Py_INCREF(Py_None);
+
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
-
-host_end:
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
 
     Py_DECREF(callback);
     PyMem_Free(data);
@@ -98,7 +111,7 @@ nameinfo_cb(void *arg, int status, int timeouts, char *node, char *service)
     PyGILState_STATE gstate = PyGILState_Ensure();
     ares_cb_data_t *data;
     DNSResolver *self;
-    PyObject *callback, *dns_status, *dns_timeouts, *dns_node, *dns_service, *result;
+    PyObject *callback, *errorno, *dns_node, *dns_service, *dns_result, *result;
 
     ASSERT(arg);
 
@@ -110,36 +123,44 @@ nameinfo_cb(void *arg, int status, int timeouts, char *node, char *service)
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    dns_status = PyLong_FromLong((long) status);
-    dns_timeouts = PyLong_FromLong((long) timeouts);
-
-    if (node) {
-        dns_node = PyString_FromString(node);
-    } else {
+    if (status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)status);
+        dns_result = Py_None;
         Py_INCREF(Py_None);
-        dns_node = Py_None;
+        goto callback;
     }
 
-    if (service) {
-        dns_service = PyString_FromString(service);
-    } else {
-        Py_INCREF(Py_None);
-        dns_service = Py_None;
-    }
-
-    if (!(dns_status && dns_timeouts && dns_node && dns_service)) {
+    dns_result = PyTuple_New(2);
+    if (!dns_result) {
         PyErr_NoMemory();
-        PyErr_WriteUnraisable(callback);
-        goto nameinfo_end;
+        PyErr_WriteUnraisable(Py_None);
+        errorno = PyInt_FromLong((long)ARES_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
     }
 
-    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_timeouts, dns_node, dns_service, NULL);
+    dns_node = PYUVString_FromString(node);
+    if (service) {
+        dns_service = PYUVString_FromString(service);
+    } else {
+        dns_service = Py_None;
+        Py_INCREF(Py_None);
+    }
+
+    PyTuple_SET_ITEM(dns_result, 0, dns_node);
+    PyTuple_SET_ITEM(dns_result, 1, dns_service);
+    errorno = Py_None;
+    Py_INCREF(Py_None);
+
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
-
-nameinfo_end:
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
 
     Py_DECREF(callback);
     PyMem_Free(data);
@@ -190,8 +211,9 @@ getaddrinfo_cb(uv_getaddrinfo_t* handle, int status, struct addrinfo* res)
     PyGILState_STATE gstate = PyGILState_Ensure();
     struct addrinfo *ptr;
     ares_cb_data_t *data;
+    uv_err_t err;
     DNSResolver *self;
-    PyObject *callback, *addr, *item, *dns_status, *dns_result, *result;
+    PyObject *callback, *addr, *item, *errorno, *dns_result, *result;
 
     ASSERT(handle);
 
@@ -203,13 +225,22 @@ getaddrinfo_cb(uv_getaddrinfo_t* handle, int status, struct addrinfo* res)
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    dns_status = PyInt_FromLong(status);
-    dns_result = PyList_New(0);
+    if (status != UV_OK) {
+        err = uv_last_error(UV_LOOP(self));
+        errorno = PyInt_FromLong((long)err.code);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
 
-    if (!(dns_status && dns_result)) {
+    dns_result = PyList_New(0);
+    if (!dns_result) {
         PyErr_NoMemory();
-        PyErr_WriteUnraisable(callback);
-        goto getaddrinfo_end;
+        PyErr_WriteUnraisable(Py_None);
+        errorno = PyInt_FromLong((long)UV_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
     }
 
     for (ptr = res; ptr; ptr = ptr->ai_next) {
@@ -217,7 +248,7 @@ getaddrinfo_cb(uv_getaddrinfo_t* handle, int status, struct addrinfo* res)
         if (!addr) {
             PyErr_NoMemory();
             PyErr_WriteUnraisable(callback);
-            goto getaddrinfo_end;
+            break;
         }
 
         item = Py_BuildValue("iiisO", ptr->ai_family, ptr->ai_socktype, ptr->ai_protocol, ptr->ai_canonname ? ptr->ai_canonname : "", addr);
@@ -225,22 +256,23 @@ getaddrinfo_cb(uv_getaddrinfo_t* handle, int status, struct addrinfo* res)
         if (!item) {
             PyErr_NoMemory();
             PyErr_WriteUnraisable(callback);
-            goto getaddrinfo_end;
+            break;
         }
 
-        if (PyList_Append(dns_result, item)) {
-            goto getaddrinfo_end;
-        }
+        PyList_Append(dns_result, item);
         Py_DECREF(item);
     }
+    errorno = Py_None;
+    Py_INCREF(Py_None);
 
-    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_result, NULL);
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
-
-getaddrinfo_end:
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
 
     Py_DECREF(callback);
     uv_freeaddrinfo(res);
@@ -262,7 +294,7 @@ query_a_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int an
     struct hostent *hostent;
     ares_cb_data_t *data;
     DNSResolver *self;
-    PyObject *dns_result, *dns_status, *tmp, *result, *callback;
+    PyObject *dns_result, *errorno, *tmp, *result, *callback;
 
     ASSERT(arg);
 
@@ -274,35 +306,52 @@ query_a_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int an
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    dns_status = PyInt_FromLong((long)status);
-    dns_result = PyList_New(0);
-
     if (status != ARES_SUCCESS) {
-        dns_status = PyInt_FromLong((long)status);
-    } else {
-        parse_status = ares_parse_a_reply(answer_buf, answer_len, &hostent, NULL, NULL);
-        if (parse_status != ARES_SUCCESS) {
-            dns_status = PyInt_FromLong((long)parse_status);
-        } else {
-            dns_status = PyInt_FromLong((long)status);
-            for (ptr = hostent->h_addr_list; *ptr != NULL; ptr++) {
-                uv_inet_ntop(hostent->h_addrtype, *ptr, ip, sizeof(ip));
-                tmp = PyString_FromString(ip);
-                if (tmp == NULL) {
-                    break;
-                }
-                PyList_Append(dns_result, tmp);
-                Py_DECREF(tmp);
-            }
-            ares_free_hostent(hostent);
-        }
+        errorno = PyInt_FromLong((long)status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
     }
 
-    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_result, NULL);
+    parse_status = ares_parse_a_reply(answer_buf, answer_len, &hostent, NULL, NULL);
+    if (parse_status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)parse_status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    dns_result = PyList_New(0);
+    if (!dns_result) {
+        PyErr_NoMemory();
+        PyErr_WriteUnraisable(Py_None);
+        errorno = PyInt_FromLong((long)ARES_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    for (ptr = hostent->h_addr_list; *ptr != NULL; ptr++) {
+        uv_inet_ntop(hostent->h_addrtype, *ptr, ip, sizeof(ip));
+        tmp = PYUVString_FromString(ip);
+        if (tmp == NULL) {
+            break;
+        }
+        PyList_Append(dns_result, tmp);
+        Py_DECREF(tmp);
+    }
+    ares_free_hostent(hostent);
+    errorno = Py_None;
+    Py_INCREF(Py_None);
+
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
 
     Py_DECREF(callback);
     PyMem_Free(data);
@@ -322,7 +371,7 @@ query_aaaa_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int
     struct hostent *hostent;
     ares_cb_data_t *data;
     DNSResolver *self;
-    PyObject *dns_result, *dns_status, *tmp, *result, *callback;
+    PyObject *dns_result, *errorno, *tmp, *result, *callback;
 
     ASSERT(arg);
 
@@ -334,35 +383,52 @@ query_aaaa_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    dns_status = PyInt_FromLong((long)status);
-    dns_result = PyList_New(0);
-
     if (status != ARES_SUCCESS) {
-        dns_status = PyInt_FromLong((long)status);
-    } else {
-        parse_status = ares_parse_aaaa_reply(answer_buf, answer_len, &hostent, NULL, NULL);
-        if (parse_status != ARES_SUCCESS) {
-            dns_status = PyInt_FromLong((long)parse_status);
-        } else {
-            dns_status = PyInt_FromLong((long)status);
-            for (ptr = hostent->h_addr_list; *ptr != NULL; ptr++) {
-                uv_inet_ntop(hostent->h_addrtype, *ptr, ip, sizeof(ip));
-                tmp = PyString_FromString(ip);
-                if (tmp == NULL) {
-                    break;
-                }
-                PyList_Append(dns_result, tmp);
-                Py_DECREF(tmp);
-            }
-            ares_free_hostent(hostent);
-        }
+        errorno = PyInt_FromLong((long)status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
     }
 
-    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_result, NULL);
+    parse_status = ares_parse_aaaa_reply(answer_buf, answer_len, &hostent, NULL, NULL);
+    if (parse_status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)parse_status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    dns_result = PyList_New(0);
+    if (!dns_result) {
+        PyErr_NoMemory();
+        PyErr_WriteUnraisable(Py_None);
+        errorno = PyInt_FromLong((long)ARES_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    for (ptr = hostent->h_addr_list; *ptr != NULL; ptr++) {
+        uv_inet_ntop(hostent->h_addrtype, *ptr, ip, sizeof(ip));
+        tmp = PYUVString_FromString(ip);
+        if (tmp == NULL) {
+            break;
+        }
+        PyList_Append(dns_result, tmp);
+        Py_DECREF(tmp);
+    }
+    ares_free_hostent(hostent);
+    errorno = Py_None;
+    Py_INCREF(Py_None);
+
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
 
     Py_DECREF(callback);
     PyMem_Free(data);
@@ -380,7 +446,7 @@ query_cname_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, in
     struct hostent *hostent;
     ares_cb_data_t *data;
     DNSResolver *self;
-    PyObject *dns_result, *dns_status, *tmp, *result, *callback;
+    PyObject *dns_result, *errorno, *tmp, *result, *callback;
 
     ASSERT(arg);
 
@@ -392,29 +458,46 @@ query_cname_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, in
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    dns_status = PyInt_FromLong((long)status);
-    dns_result = PyList_New(0);
-
     if (status != ARES_SUCCESS) {
-        dns_status = PyInt_FromLong((long)status);
-    } else {
-        parse_status = ares_parse_a_reply(answer_buf, answer_len, &hostent, NULL, NULL);
-        if (parse_status != ARES_SUCCESS) {
-            dns_status = PyInt_FromLong((long)parse_status);
-        } else {
-            dns_status = PyInt_FromLong((long)status);
-            tmp = PyString_FromString(hostent->h_name);
-            PyList_Append(dns_result, tmp);
-            Py_DECREF(tmp);
-            ares_free_hostent(hostent);
-        }
+        errorno = PyInt_FromLong((long)status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
     }
 
-    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_result, NULL);
+    parse_status = ares_parse_a_reply(answer_buf, answer_len, &hostent, NULL, NULL);
+    if (parse_status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)parse_status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    dns_result = PyList_New(0);
+    if (!dns_result) {
+        PyErr_NoMemory();
+        PyErr_WriteUnraisable(Py_None);
+        errorno = PyInt_FromLong((long)ARES_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    tmp = PYUVString_FromString(hostent->h_name);
+    PyList_Append(dns_result, tmp);
+    Py_DECREF(tmp);
+    ares_free_hostent(hostent);
+    errorno = Py_None;
+    Py_INCREF(Py_None);
+
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
 
     Py_DECREF(callback);
     PyMem_Free(data);
@@ -432,7 +515,7 @@ query_mx_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int a
     struct ares_mx_reply *mx_reply, *mx_ptr;
     ares_cb_data_t *data;
     DNSResolver *self;
-    PyObject *dns_result, *dns_status, *tmp, *result, *callback;
+    PyObject *dns_result, *errorno, *tmp, *result, *callback;
 
     ASSERT(arg);
 
@@ -444,36 +527,53 @@ query_mx_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int a
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    dns_status = PyInt_FromLong((long)status);
-    dns_result = PyList_New(0);
-
     if (status != ARES_SUCCESS) {
-        dns_status = PyInt_FromLong((long)status);
-    } else {
-        parse_status = ares_parse_mx_reply(answer_buf, answer_len, &mx_reply);
-        if (parse_status != ARES_SUCCESS) {
-            dns_status = PyInt_FromLong((long)parse_status);
-        } else {
-            dns_status = PyInt_FromLong((long)status);
-            for (mx_ptr = mx_reply; mx_ptr != NULL; mx_ptr = mx_ptr->next) {
-                tmp = PyTuple_New(2);
-                if (tmp == NULL) {
-                    break;
-                }
-                PyTuple_SET_ITEM(tmp, 0, PyString_FromString(mx_ptr->host));
-                PyTuple_SET_ITEM(tmp, 1, PyInt_FromLong((long)mx_ptr->priority));
-                PyList_Append(dns_result, tmp);
-                Py_DECREF(tmp);
-            }
-            ares_free_data(mx_reply);
-        }
+        errorno = PyInt_FromLong((long)status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
     }
 
-    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_result, NULL);
+    parse_status = ares_parse_mx_reply(answer_buf, answer_len, &mx_reply);
+    if (parse_status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)parse_status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    dns_result = PyList_New(0);
+    if (!dns_result) {
+        PyErr_NoMemory();
+        PyErr_WriteUnraisable(Py_None);
+        errorno = PyInt_FromLong((long)ARES_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    for (mx_ptr = mx_reply; mx_ptr != NULL; mx_ptr = mx_ptr->next) {
+        tmp = PyTuple_New(2);
+        if (tmp == NULL) {
+            break;
+        }
+        PyTuple_SET_ITEM(tmp, 0, PYUVString_FromString(mx_ptr->host));
+        PyTuple_SET_ITEM(tmp, 1, PyInt_FromLong((long)mx_ptr->priority));
+        PyList_Append(dns_result, tmp);
+        Py_DECREF(tmp);
+    }
+    ares_free_data(mx_reply);
+    errorno = Py_None;
+    Py_INCREF(Py_None);
+
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
 
     Py_DECREF(callback);
     PyMem_Free(data);
@@ -492,7 +592,7 @@ query_ns_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int a
     struct hostent *hostent;
     ares_cb_data_t *data;
     DNSResolver *self;
-    PyObject *dns_result, *dns_status, *tmp, *result, *callback;
+    PyObject *dns_result, *errorno, *tmp, *result, *callback;
 
     ASSERT(arg);
 
@@ -504,34 +604,51 @@ query_ns_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int a
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    dns_status = PyInt_FromLong((long)status);
-    dns_result = PyList_New(0);
-
     if (status != ARES_SUCCESS) {
-        dns_status = PyInt_FromLong((long)status);
-    } else {
-        parse_status = ares_parse_ns_reply(answer_buf, answer_len, &hostent);
-        if (parse_status != ARES_SUCCESS) {
-            dns_status = PyInt_FromLong((long)parse_status);
-        } else {
-            dns_status = PyInt_FromLong((long)status);
-            for (ptr = hostent->h_aliases; *ptr != NULL; ptr++) {
-                tmp = PyString_FromString(*ptr);
-                if (tmp == NULL) {
-                    break;
-                }
-                PyList_Append(dns_result, tmp);
-                Py_DECREF(tmp);
-            }
-            ares_free_hostent(hostent);
-        }
+        errorno = PyInt_FromLong((long)status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
     }
 
-    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_result, NULL);
+    parse_status = ares_parse_ns_reply(answer_buf, answer_len, &hostent);
+    if (parse_status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)parse_status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    dns_result = PyList_New(0);
+    if (!dns_result) {
+        PyErr_NoMemory();
+        PyErr_WriteUnraisable(Py_None);
+        errorno = PyInt_FromLong((long)ARES_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    for (ptr = hostent->h_aliases; *ptr != NULL; ptr++) {
+        tmp = PYUVString_FromString(*ptr);
+        if (tmp == NULL) {
+            break;
+        }
+        PyList_Append(dns_result, tmp);
+        Py_DECREF(tmp);
+    }
+    ares_free_hostent(hostent);
+    errorno = Py_None;
+    Py_INCREF(Py_None);
+
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
 
     Py_DECREF(callback);
     PyMem_Free(data);
@@ -549,7 +666,7 @@ query_txt_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int
     struct ares_txt_reply *txt_reply, *txt_ptr;
     ares_cb_data_t *data;
     DNSResolver *self;
-    PyObject *dns_result, *dns_status, *tmp, *result, *callback;
+    PyObject *dns_result, *errorno, *tmp, *result, *callback;
 
     ASSERT(arg);
 
@@ -561,34 +678,51 @@ query_txt_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    dns_status = PyInt_FromLong((long)status);
-    dns_result = PyList_New(0);
-
     if (status != ARES_SUCCESS) {
-        dns_status = PyInt_FromLong((long)status);
-    } else {
-        parse_status = ares_parse_txt_reply(answer_buf, answer_len, &txt_reply);
-        if (parse_status != ARES_SUCCESS) {
-            dns_status = PyInt_FromLong((long)parse_status);
-        } else {
-            dns_status = PyInt_FromLong((long)status);
-            for (txt_ptr = txt_reply; txt_ptr != NULL; txt_ptr = txt_ptr->next) {
-                tmp = PyString_FromString((const char *)txt_ptr->txt);
-                if (tmp == NULL) {
-                    break;
-                }
-                PyList_Append(dns_result, tmp);
-                Py_DECREF(tmp);
-            }
-            ares_free_data(txt_reply);
-        }
+        errorno = PyInt_FromLong((long)status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
     }
 
-    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_result, NULL);
+    parse_status = ares_parse_txt_reply(answer_buf, answer_len, &txt_reply);
+    if (parse_status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)parse_status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    dns_result = PyList_New(0);
+    if (!dns_result) {
+        PyErr_NoMemory();
+        PyErr_WriteUnraisable(Py_None);
+        errorno = PyInt_FromLong((long)ARES_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    for (txt_ptr = txt_reply; txt_ptr != NULL; txt_ptr = txt_ptr->next) {
+        tmp = PYUVString_FromString((const char *)txt_ptr->txt);
+        if (tmp == NULL) {
+            break;
+        }
+        PyList_Append(dns_result, tmp);
+        Py_DECREF(tmp);
+    }
+    ares_free_data(txt_reply);
+    errorno = Py_None;
+    Py_INCREF(Py_None);
+
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
 
     Py_DECREF(callback);
     PyMem_Free(data);
@@ -606,7 +740,7 @@ query_srv_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int
     struct ares_srv_reply *srv_reply, *srv_ptr;
     ares_cb_data_t *data;
     DNSResolver *self;
-    PyObject *dns_result, *dns_status, *tmp, *result, *callback;
+    PyObject *dns_result, *errorno, *tmp, *result, *callback;
 
     ASSERT(arg);
 
@@ -618,38 +752,135 @@ query_srv_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    dns_status = PyInt_FromLong((long)status);
-    dns_result = PyList_New(0);
-
     if (status != ARES_SUCCESS) {
-        dns_status = PyInt_FromLong((long)status);
-    } else {
-        parse_status = ares_parse_srv_reply(answer_buf, answer_len, &srv_reply);
-        if (parse_status != ARES_SUCCESS) {
-            dns_status = PyInt_FromLong((long)parse_status);
-        } else {
-            dns_status = PyInt_FromLong((long)status);
-            for (srv_ptr = srv_reply; srv_ptr != NULL; srv_ptr = srv_ptr->next) {
-                tmp = PyTuple_New(4);
-                if (tmp == NULL) {
-                    break;
-                }
-                PyTuple_SET_ITEM(tmp, 0, PyString_FromString(srv_ptr->host));
-                PyTuple_SET_ITEM(tmp, 1, PyInt_FromLong((long)srv_ptr->port));
-                PyTuple_SET_ITEM(tmp, 2, PyInt_FromLong((long)srv_ptr->priority));
-                PyTuple_SET_ITEM(tmp, 3, PyInt_FromLong((long)srv_ptr->weight));
-                PyList_Append(dns_result, tmp);
-                Py_DECREF(tmp);
-            }
-            ares_free_data(srv_reply);
-        }
+        errorno = PyInt_FromLong((long)status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
     }
 
-    result = PyObject_CallFunctionObjArgs(callback, self, dns_status, dns_result, NULL);
+    parse_status = ares_parse_srv_reply(answer_buf, answer_len, &srv_reply);
+    if (parse_status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)parse_status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    dns_result = PyList_New(0);
+    if (!dns_result) {
+        PyErr_NoMemory();
+        PyErr_WriteUnraisable(Py_None);
+        errorno = PyInt_FromLong((long)ARES_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    for (srv_ptr = srv_reply; srv_ptr != NULL; srv_ptr = srv_ptr->next) {
+        tmp = PyTuple_New(4);
+        if (tmp == NULL) {
+            break;
+        }
+        PyTuple_SET_ITEM(tmp, 0, PYUVString_FromString(srv_ptr->host));
+        PyTuple_SET_ITEM(tmp, 1, PyInt_FromLong((long)srv_ptr->port));
+        PyTuple_SET_ITEM(tmp, 2, PyInt_FromLong((long)srv_ptr->priority));
+        PyTuple_SET_ITEM(tmp, 3, PyInt_FromLong((long)srv_ptr->weight));
+        PyList_Append(dns_result, tmp);
+        Py_DECREF(tmp);
+    }
+    ares_free_data(srv_reply);
+    errorno = Py_None;
+    Py_INCREF(Py_None);
+
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
     if (result == NULL) {
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
+
+    Py_DECREF(callback);
+    PyMem_Free(data);
+
+    Py_DECREF(self);
+    PyGILState_Release(gstate);
+}
+
+
+static void
+query_naptr_cb(void *arg, int status,int timeouts, unsigned char *answer_buf, int answer_len)
+{
+    PyGILState_STATE gstate = PyGILState_Ensure();
+    int parse_status;
+    struct ares_naptr_reply *naptr_reply, *naptr_ptr;
+    ares_cb_data_t *data;
+    DNSResolver *self;
+    PyObject *dns_result, *errorno, *tmp, *result, *callback;
+
+    ASSERT(arg);
+
+    data = (ares_cb_data_t*)arg;
+    self = data->resolver;
+    callback = data->cb;
+
+    ASSERT(self);
+    /* Object could go out of scope in the callback, increase refcount to avoid it */
+    Py_INCREF(self);
+
+    if (status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    parse_status = ares_parse_naptr_reply(answer_buf, answer_len, &naptr_reply);
+    if (parse_status != ARES_SUCCESS) {
+        errorno = PyInt_FromLong((long)parse_status);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    dns_result = PyList_New(0);
+    if (!dns_result) {
+        PyErr_NoMemory();
+        PyErr_WriteUnraisable(Py_None);
+        errorno = PyInt_FromLong((long)ARES_ENOMEM);
+        dns_result = Py_None;
+        Py_INCREF(Py_None);
+        goto callback;
+    }
+
+    for (naptr_ptr = naptr_reply; naptr_ptr != NULL; naptr_ptr = naptr_ptr->next) {
+        tmp = PyTuple_New(6);
+        if (tmp == NULL) {
+            break;
+        }
+        PyTuple_SET_ITEM(tmp, 0, PyInt_FromLong((long)naptr_ptr->order));
+        PyTuple_SET_ITEM(tmp, 1, PyInt_FromLong((long)naptr_ptr->preference));
+        PyTuple_SET_ITEM(tmp, 2, PYUVString_FromString((char *)naptr_ptr->flags));
+        PyTuple_SET_ITEM(tmp, 3, PYUVString_FromString((char *)naptr_ptr->service));
+        PyTuple_SET_ITEM(tmp, 4, PYUVString_FromString((char *)naptr_ptr->regexp));
+        PyTuple_SET_ITEM(tmp, 5, PYUVString_FromString(naptr_ptr->replacement));
+        PyList_Append(dns_result, tmp);
+        Py_DECREF(tmp);
+    }
+    ares_free_data(naptr_reply);
+    errorno = Py_None;
+    Py_INCREF(Py_None);
+
+callback:
+    result = PyObject_CallFunctionObjArgs(callback, self, dns_result, errorno, NULL);
+    if (result == NULL) {
+        PyErr_WriteUnraisable(callback);
+    }
+    Py_XDECREF(result);
+    Py_DECREF(dns_result);
+    Py_DECREF(errorno);
 
     Py_DECREF(callback);
     PyMem_Free(data);
@@ -663,13 +894,10 @@ static PyObject *
 DNSResolver_func_gethostbyname(DNSResolver *self, PyObject *args, PyObject *kwargs)
 {
     char *name;
-    int family = AF_INET;
     ares_cb_data_t *cb_data;
     PyObject *callback;
 
-    static char *kwlist[] = {"callback", "name", "family", NULL};
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "Os|i:gethostbyname", kwlist, &callback, &name, &family)) {
+    if (!PyArg_ParseTuple(args, "sO:gethostbyname", &name, &callback)) {
         return NULL;
     }
 
@@ -687,7 +915,7 @@ DNSResolver_func_gethostbyname(DNSResolver *self, PyObject *args, PyObject *kwar
     cb_data->resolver = self;
     cb_data->cb = callback;
 
-    ares_gethostbyname(self->channel, name, family, &host_cb, (void *)cb_data);
+    ares_gethostbyname(self->channel, name, AF_INET, &host_cb, (void *)cb_data);
 
     Py_RETURN_NONE;
 }
@@ -704,9 +932,7 @@ DNSResolver_func_gethostbyaddr(DNSResolver *self, PyObject *args, PyObject *kwar
     ares_cb_data_t *cb_data;
     PyObject *callback;
 
-    static char *kwlist[] = {"callback", "name", NULL};
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "Os:gethostbyaddr", kwlist, &callback, &name)) {
+    if (!PyArg_ParseTuple(args, "sO:gethostbyaddr", &name, &callback)) {
         return NULL;
     }
 
@@ -746,7 +972,7 @@ DNSResolver_func_gethostbyaddr(DNSResolver *self, PyObject *args, PyObject *kwar
 static PyObject *
 DNSResolver_func_getnameinfo(DNSResolver *self, PyObject *args, PyObject *kwargs)
 {
-    char *name;
+    char *addr;
     int port, flags, length;
     struct in_addr addr4;
     struct in6_addr addr6;
@@ -756,11 +982,7 @@ DNSResolver_func_getnameinfo(DNSResolver *self, PyObject *args, PyObject *kwargs
     ares_cb_data_t *cb_data;
     PyObject *callback;
 
-    static char *kwlist[] = {"callback", "name", "port", "flags", NULL};
-
-    port = flags = 0;
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "Os|ii:getnameinfo", kwlist, &callback, &name, &port, &flags)) {
+    if (!PyArg_ParseTuple(args, "(si)iO:getnameinfo", &addr, &port, &flags, &callback)) {
         return NULL;
     }
 
@@ -774,12 +996,12 @@ DNSResolver_func_getnameinfo(DNSResolver *self, PyObject *args, PyObject *kwargs
         return NULL;
     }
 
-    if (uv_inet_pton(AF_INET, name, &addr4) == 1) {
-        sa4 = uv_ip4_addr(name, port);
+    if (uv_inet_pton(AF_INET, addr, &addr4) == 1) {
+        sa4 = uv_ip4_addr(addr, port);
         sa = (struct sockaddr *)&sa4;
         length = sizeof(struct sockaddr_in);
-    } else if (uv_inet_pton(AF_INET6, name, &addr6) == 1) {
-        sa6 = uv_ip6_addr(name, port);
+    } else if (uv_inet_pton(AF_INET6, addr, &addr6) == 1) {
+        sa6 = uv_ip6_addr(addr, port);
         sa = (struct sockaddr *)&sa6;
         length = sizeof(struct sockaddr_in6);
     } else {
@@ -817,7 +1039,7 @@ DNSResolver_func_getaddrinfo(DNSResolver *self, PyObject *args, PyObject *kwargs
     port = socktype = protocol = flags = 0;
     family = AF_UNSPEC;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "Os|iiiii:getaddrinfo", kwlist, &callback, &name, &port, &family, &socktype, &protocol, &flags)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "sO|iiiii:getaddrinfo", kwlist, &name, &callback, &port, &family, &socktype, &protocol, &flags)) {
         return NULL;
     }
 
@@ -1092,6 +1314,45 @@ DNSResolver_func_query_srv(DNSResolver *self, PyObject *args)
 }
 
 
+static PyObject *
+DNSResolver_func_query_naptr(DNSResolver *self, PyObject *args)
+{
+    char *name;
+    ares_cb_data_t *cb_data;
+    PyObject *callback;
+
+    if (!PyArg_ParseTuple(args, "sO:query_naptr", &name, &callback)) {
+        return NULL;
+    }
+
+    if (!PyCallable_Check(callback)) {
+        PyErr_SetString(PyExc_TypeError, "a callable is required");
+        return NULL;
+    }
+
+    cb_data = (ares_cb_data_t*) PyMem_Malloc(sizeof *cb_data);
+    if (!cb_data) {
+        return PyErr_NoMemory();
+    }
+
+    Py_INCREF(callback);
+    cb_data->resolver = self;
+    cb_data->cb = callback;
+
+    ares_query(self->channel, name, C_IN, T_NAPTR, &query_naptr_cb, (void *)cb_data);
+
+    Py_RETURN_NONE;
+}
+
+
+static PyObject *
+DNSResolver_func_cancel(DNSResolver *self)
+{
+    ares_cancel(self->channel);
+    Py_RETURN_NONE;
+}
+
+
 static int
 set_dns_servers(DNSResolver *self, PyObject *value)
 {
@@ -1184,10 +1445,10 @@ DNSResolver_servers_get(DNSResolver *self, void *closure)
     for (server = servers; server != NULL; server = server->next) {
         if (server->family == AF_INET) {
             uv_inet_ntop(AF_INET, &(server->addr.addr4), ip, INET_ADDRSTRLEN);
-            tmp = PyString_FromString(ip);
+            tmp = PYUVString_FromString(ip);
         } else {
             uv_inet_ntop(AF_INET6, &(server->addr.addr6), ip, INET6_ADDRSTRLEN);
-            tmp = PyString_FromString(ip);
+            tmp = PYUVString_FromString(ip);
         }
         if (tmp == NULL) {
             break;
@@ -1300,6 +1561,7 @@ DNSResolver_tp_dealloc(DNSResolver *self)
 
 static PyMethodDef
 DNSResolver_tp_methods[] = {
+    { "cancel", (PyCFunction)DNSResolver_func_cancel, METH_NOARGS, "Cancel all pending queries on this resolver" },
     { "gethostbyname", (PyCFunction)DNSResolver_func_gethostbyname, METH_VARARGS|METH_KEYWORDS, "Gethostbyname" },
     { "gethostbyaddr", (PyCFunction)DNSResolver_func_gethostbyaddr, METH_VARARGS|METH_KEYWORDS, "Gethostbyaddr" },
     { "getnameinfo", (PyCFunction)DNSResolver_func_getnameinfo, METH_VARARGS|METH_KEYWORDS, "Getnameinfo" },
@@ -1311,6 +1573,7 @@ DNSResolver_tp_methods[] = {
     { "query_ns", (PyCFunction)DNSResolver_func_query_ns, METH_VARARGS, "Run a DNS query of type NS" },
     { "query_txt", (PyCFunction)DNSResolver_func_query_txt, METH_VARARGS, "Run a DNS query of type TXT" },
     { "query_srv", (PyCFunction)DNSResolver_func_query_srv, METH_VARARGS, "Run a DNS query of type SRV" },
+    { "query_naptr", (PyCFunction)DNSResolver_func_query_naptr, METH_VARARGS, "Run a DNS query of type NAPTR" },
     { NULL }
 };
 
