@@ -41,8 +41,24 @@ Errno_func_strerror(PyObject *obj, PyObject *args)
 }
 
 
+static PyObject *
+Errno_func_ares_strerror(PyObject *obj, PyObject *args)
+{
+    int errorno;
+
+    UNUSED_ARG(obj);
+
+    if (!PyArg_ParseTuple(args, "i:ares_strerror", &errorno)) {
+        return NULL;
+    }
+
+    return PyString_FromString(ares_strerror(errorno));
+}
+
+
 static PyMethodDef
 Errno_methods[] = {
+    { "ares_strerror", (PyCFunction)Errno_func_ares_strerror, METH_VARARGS, "Get string representation of a c-ares error code." },
     { "strerror", (PyCFunction)Errno_func_strerror, METH_VARARGS, "Get string representation of an error code." },
     { NULL }
 };
@@ -131,6 +147,11 @@ init_errno(void)
     inscode(module_dict, errorcode_dict, "UV_EEXIST", UV_EEXIST);
     inscode(module_dict, errorcode_dict, "UV_ESRCH", UV_ESRCH);
     inscode(module_dict, errorcode_dict, "UV_ENAMETOOLONG", UV_ENAMETOOLONG);
+    inscode(module_dict, errorcode_dict, "UV_EPERM", UV_EPERM);
+    inscode(module_dict, errorcode_dict, "UV_ELOOP", UV_ELOOP);
+    inscode(module_dict, errorcode_dict, "UV_EXDEV", UV_EXDEV);
+    inscode(module_dict, errorcode_dict, "UV_ENOTEMPTY", UV_ENOTEMPTY);
+    inscode(module_dict, errorcode_dict, "UV_ENOSPC", UV_ENOSPC);
 
     inscode(module_dict, ares_errorcode_dict, "ARES_SUCCESS", ARES_SUCCESS);
     inscode(module_dict, ares_errorcode_dict, "ARES_ENODATA", ARES_ENODATA);
