@@ -39,11 +39,7 @@ process_stat(uv_fs_t* req, PyObject **path, PyObject **stat_data, PyObject **err
     st = (struct stat *)(req->ptr);
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        *path = PyUnicode_FromString(req->path);
-#else
-        *path = PyString_FromString(req->path);
-#endif
+        *path = PYUVString_FromString(req->path);
     } else {
         *path = Py_None;
         Py_INCREF(Py_None);
@@ -143,6 +139,9 @@ stat_cb(uv_fs_t* req) {
             PyErr_WriteUnraisable(req_data->callback);
         }
         Py_XDECREF(result);
+        Py_DECREF(stat_data);
+        Py_DECREF(path);
+        Py_DECREF(errorno);
     }
 
     uv_fs_req_cleanup(req);
@@ -167,11 +166,7 @@ unlink_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)req->data;
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif  
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -189,6 +184,8 @@ unlink_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -212,11 +209,7 @@ mkdir_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)req->data;
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif  
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -234,6 +227,8 @@ mkdir_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -257,11 +252,7 @@ rmdir_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)req->data;
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif  
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -279,6 +270,8 @@ rmdir_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -302,11 +295,7 @@ rename_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)req->data;
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif  
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -324,6 +313,8 @@ rename_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -347,11 +338,7 @@ chmod_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)req->data;
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif  
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -369,6 +356,8 @@ chmod_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -392,11 +381,7 @@ link_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)req->data;
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif  
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -414,6 +399,8 @@ link_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -437,11 +424,7 @@ symlink_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif  
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -459,6 +442,8 @@ symlink_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -483,11 +468,7 @@ process_readlink(uv_fs_t* req, PyObject **path, PyObject **errorno)
     } else {
         *errorno = Py_None;
         Py_INCREF(Py_None);
-#ifdef PYUV_PYTHON3
-        *path = PyUnicode_FromString(req->ptr);
-#else
-        *path = PyString_FromString(req->ptr);
-#endif
+        *path = PYUVString_FromString(req->ptr);
     }
 }
 
@@ -506,6 +487,8 @@ readlink_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -529,11 +512,7 @@ chown_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)req->data;
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif  
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -551,6 +530,8 @@ chown_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -569,11 +550,7 @@ process_open(uv_fs_t* req, PyObject **path, PyObject **fd, PyObject **errorno)
     ASSERT(req->fs_type == UV_FS_OPEN);
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        *path = PyUnicode_FromString(req->path);
-#else
-        *path = PyString_FromString(req->path);
-#endif  
+        *path = PYUVString_FromString(req->path);
     } else {
         *path = Py_None;
         Py_INCREF(Py_None);
@@ -605,6 +582,9 @@ open_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(fd);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -627,11 +607,7 @@ close_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif  
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -649,6 +625,8 @@ close_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -671,11 +649,7 @@ process_read(uv_fs_t* req, PyObject **path, PyObject **read_data, PyObject **err
     req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        *path = PyUnicode_FromString(req->path);
-#else
-        *path = PyString_FromString(req->path);
-#endif
+        *path = PYUVString_FromString(req->path);
     } else {
         *path = Py_None;
         Py_INCREF(Py_None);
@@ -688,11 +662,7 @@ process_read(uv_fs_t* req, PyObject **path, PyObject **read_data, PyObject **err
     } else {
         *errorno = Py_None;
         Py_INCREF(Py_None);
-#ifdef PYUV_PYTHON3
-        *read_data = PyUnicode_FromStringAndSize(req_data->buf.base, req->result);
-#else
-        *read_data = PyString_FromStringAndSize(req_data->buf.base, req->result);
-#endif
+        *read_data = PYUVString_FromStringAndSize(req_data->buf.base, req->result);
     }
 }
 
@@ -711,6 +681,9 @@ read_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(read_data);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -731,11 +704,7 @@ process_write(uv_fs_t* req, PyObject **path, PyObject **bytes_written, PyObject
 
     *bytes_written = PyInt_FromLong((long)req->result);
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        *path = PyUnicode_FromString(req->path);
-#else
-        *path = PyString_FromString(req->path);
-#endif
+        *path = PYUVString_FromString(req->path);
     } else {
         *path = Py_None;
         Py_INCREF(Py_None);
@@ -767,6 +736,9 @@ write_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(bytes_written);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -791,11 +763,7 @@ fsync_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -813,6 +781,8 @@ fsync_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -836,11 +806,7 @@ ftruncate_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -858,6 +824,8 @@ ftruncate_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -880,11 +848,7 @@ process_readdir(uv_fs_t* req, PyObject **path, PyObject **files, PyObject **erro
     ASSERT(req->fs_type == UV_FS_READDIR);
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        *path = PyUnicode_FromString(req->path);
-#else
-        *path = PyString_FromString(req->path);
-#endif
+        *path = PYUVString_FromString(req->path);
     } else {
         *path = Py_None;
         Py_INCREF(Py_None);
@@ -907,11 +871,7 @@ process_readdir(uv_fs_t* req, PyObject **path, PyObject **files, PyObject **erro
             r = req->result;
             ptr = req->ptr;
             while (r--) {
-#ifdef PYUV_PYTHON3
-                item = PyUnicode_FromString(ptr);
-#else
-                item = PyString_FromString(ptr);
-#endif
+                item = PYUVString_FromString(ptr);
                 PyList_Append(*files, item);
                 Py_DECREF(item);
                 ptr += strlen(ptr) + 1;
@@ -936,6 +896,9 @@ readdir_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(files);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -956,11 +919,7 @@ process_sendfile(uv_fs_t* req, PyObject **path, PyObject **bytes_written, PyObje
     *bytes_written = PyInt_FromLong((long)req->result);
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        *path = PyUnicode_FromString(req->path);
-#else
-        *path = PyString_FromString(req->path);
-#endif
+        *path = PYUVString_FromString(req->path);
     } else {
         *path = Py_None;
         Py_INCREF(Py_None);
@@ -989,6 +948,9 @@ sendfile_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(bytes_written);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -1012,11 +974,7 @@ utime_cb(uv_fs_t* req) {
     req_data = (fs_req_data_t*)(req->data);
 
     if (req->path != NULL) {
-#ifdef PYUV_PYTHON3
-        path = PyUnicode_FromString(req->path);
-#else
-        path = PyString_FromString(req->path);
-#endif
+        path = PYUVString_FromString(req->path);
     } else {
         path = Py_None;
         Py_INCREF(Py_None);
@@ -1034,6 +992,8 @@ utime_cb(uv_fs_t* req) {
         PyErr_WriteUnraisable(req_data->callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(path);
+    Py_DECREF(errorno);
 
     uv_fs_req_cleanup(req);
     Py_DECREF(req_data->loop);
@@ -2153,11 +2113,13 @@ end:
 static PyObject *
 FS_func_write(PyObject *obj, PyObject *args, PyObject *kwargs)
 {
-    int r, fd, offset, write_len;
+    int r, fd, offset;
     char *write_str, *buf_data;
     uv_fs_t *fs_req = NULL;
     fs_req_data_t *req_data = NULL;
     Loop *loop;
+    Py_ssize_t write_len;
+    Py_buffer pbuf;
     PyObject *callback, *py_path, *py_errorno, *written_bytes, *ret;
 
     static char *kwlist[] = {"loop", "fd", "write_data", "offset", "callback", NULL};
@@ -2166,11 +2128,12 @@ FS_func_write(PyObject *obj, PyObject *args, PyObject *kwargs)
     write_str = buf_data = NULL;
     callback = NULL;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!is#i|O:write", kwlist, &LoopType, &loop, &fd, &write_str, &write_len, &offset, &callback)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!is*i|O:write", kwlist, &LoopType, &loop, &fd, &pbuf, &offset, &callback)) {
         return NULL;
     }
 
     if (callback != NULL && !PyCallable_Check(callback)) {
+        PyBuffer_Release(&pbuf);
         PyErr_SetString(PyExc_TypeError, "a callable is required");
         return NULL;
     }
@@ -2189,13 +2152,15 @@ FS_func_write(PyObject *obj, PyObject *args, PyObject *kwargs)
         goto end;
     }
 
-    buf_data = (char *)PyMem_Malloc(write_len+1);
+    write_str = pbuf.buf;
+    write_len = pbuf.len;
+    buf_data = (char *)PyMem_Malloc(write_len);
     if (!buf_data) {
         PyErr_NoMemory();
         ret = NULL;
         goto end;
     }
-    memcpy(buf_data, write_str, write_len+1);
+    memcpy(buf_data, write_str, write_len);
 
     Py_INCREF(loop);
     Py_XINCREF(callback);
@@ -2214,6 +2179,7 @@ FS_func_write(PyObject *obj, PyObject *args, PyObject *kwargs)
 
     if (callback != NULL) {
         /* No need to cleanup, it will be done in the callback */
+        PyBuffer_Release(&pbuf);
         Py_RETURN_NONE;
     } else {
         process_write(fs_req, &py_path, &written_bytes, &py_errorno);
@@ -2223,6 +2189,7 @@ FS_func_write(PyObject *obj, PyObject *args, PyObject *kwargs)
     }
 
 end:
+    PyBuffer_Release(&pbuf);
     if (fs_req) {
         PyMem_Free(fs_req);
     }
@@ -2796,11 +2763,7 @@ on_fsevent_callback(uv_fs_event_t *handle, const char *filename, int events, int
     Py_INCREF(self);
 
     if (filename) {
-#ifdef PYUV_PYTHON3
-        py_filename = PyUnicode_FromString(filename);
-#else
-        py_filename = PyString_FromString(filename);
-#endif
+        py_filename = PYUVString_FromString(filename);
     } else {
         py_filename = Py_None;
         Py_INCREF(Py_None);
@@ -2821,6 +2784,9 @@ on_fsevent_callback(uv_fs_event_t *handle, const char *filename, int events, int
 	PyErr_WriteUnraisable(self->on_fsevent_cb);
     }
     Py_XDECREF(result);
+    Py_DECREF(py_events);
+    Py_DECREF(py_filename);
+    Py_DECREF(errorno);
 
     Py_DECREF(self);
     PyGILState_Release(gstate);
