@@ -1,8 +1,21 @@
-/* Some helper functions */
+/* Some helper stuff */
+
+
+/* Temporary hack: libuv should provide uv_inet_pton and uv_inet_ntop. */
+#ifdef PYUV_WINDOWS
+    #include <inet_net_pton.h>
+    #include <inet_ntop.h>
+    #define uv_inet_pton ares_inet_pton
+    #define uv_inet_ntop ares_inet_ntop
+#else /* __POSIX__ */
+    #include <arpa/inet.h>
+    #define uv_inet_pton inet_pton
+    #define uv_inet_ntop inet_ntop
+#endif
 
 
 /* Add a type to a module */
-int
+static int
 PyUVModule_AddType(PyObject *module, const char *name, PyTypeObject *type)
 {
     if (PyType_Ready(type)) {
@@ -18,7 +31,7 @@ PyUVModule_AddType(PyObject *module, const char *name, PyTypeObject *type)
 
 
 /* Add a type to a module */
-int
+static int
 PyUVModule_AddObject(PyObject *module, const char *name, PyObject *value)
 {
     Py_INCREF(value);
@@ -30,8 +43,14 @@ PyUVModule_AddObject(PyObject *module, const char *name, PyObject *value)
 }
 
 
+#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
+    #define INLINE inline
+#else
+    #define INLINE
+#endif
+
 /* Raise appropriate exception when an error is produced inside libuv */
-inline void
+static INLINE void
 raise_uv_exception(Loop *loop, PyObject *exc_type)
 {
     uv_err_t err = uv_last_error(loop->uv_loop);
