@@ -31,6 +31,7 @@ on_pipe_connection(uv_stream_t* server, int status)
         PyErr_WriteUnraisable(self->on_new_connection_cb);
     }
     Py_XDECREF(result);
+    Py_DECREF(py_errorno);
 
     Py_DECREF(self);
     PyGILState_Release(gstate);
@@ -70,6 +71,7 @@ on_pipe_client_connection(uv_connect_t *req, int status)
         PyErr_WriteUnraisable(callback);
     }
     Py_XDECREF(result);
+    Py_DECREF(py_errorno);
 
     Py_DECREF(callback);
     PyMem_Free(req_data);
@@ -112,6 +114,9 @@ on_pipe_read2(uv_pipe_t* handle, int nread, uv_buf_t buf, uv_handle_type pending
         PyErr_WriteUnraisable(self->on_read_cb);
     }
     Py_XDECREF(result);
+    Py_DECREF(data);
+    Py_DECREF(py_pending);
+    Py_DECREF(py_errorno);
 
     PyMem_Free(buf.base);
 
@@ -368,12 +373,11 @@ Pipe_func_start_read2(Pipe *self, PyObject *args)
 static PyObject *
 Pipe_func_write2(Pipe *self, PyObject *args)
 {
-    int i, n, r, buf_count;
-    char *data_str;
-    char *tmp;
+    int i, r, buf_count;
+    char *data_str, *tmp;
+    Py_buffer pbuf;
     Py_ssize_t data_len;
-    PyObject *item, *data, *callback;
-    TCP *send_handle;
+    PyObject *callback, *send_handle;
     uv_buf_t tmpbuf;
     uv_buf_t *bufs = NULL;
     uv_write_t *wr = NULL;
@@ -390,16 +394,17 @@ Pipe_func_write2(Pipe *self, PyObject *args)
         return NULL;
     }
 
-    if (!PyArg_ParseTuple(args, "OO!|O:write2", &data, &TCPType, &send_handle, &callback)) {
+    if (!PyArg_ParseTuple(args, "s*O|O:write2", &pbuf, &send_handle, &callback)) {
         return NULL;
     }
 
-    if (!PySequence_Check(data) || PyUnicode_Check(data)) {
-        PyErr_SetString(PyExc_TypeError, "only strings and iterables are supported");
+    if (!PyObject_TypeCheck(send_handle, &TCPType) && !PyObject_TypeCheck(send_handle, &PipeType)) {
+        PyErr_SetString(PyExc_TypeError, "Only TCP and Pipe objects are supported for write2");
         return NULL;
     }
 
     if (callback != Py_None && !PyCallable_Check(callback)) {
+        PyBuffer_Release(&pbuf);
         PyErr_SetString(PyExc_TypeError, "a callable or None is required");
         return NULL;
     }
@@ -427,51 +432,24 @@ Pipe_func_write2(Pipe *self, PyObject *args)
     req_data->callback = callback;
     wr->data = (void *)req_data;
 
-    if (PyString_Check(data)) {
-        /* We have a single string */
-        bufs = (uv_buf_t *) PyMem_Malloc(sizeof(uv_buf_t));
-        if (!bufs) {
-            PyErr_NoMemory();
-            goto error;
-        }
-        data_len = PyString_Size(data);
-        data_str = PyString_AsString(data);
-        tmp = (char *) PyMem_Malloc(data_len + 1);
-        if (!tmp) {
-            PyMem_Free(bufs);
-            PyErr_NoMemory();
-            goto error;
-        }
-        memcpy(tmp, data_str, data_len + 1);
-        tmpbuf = uv_buf_init(tmp, data_len);
-        bufs[0] = tmpbuf;
-        buf_count = 1;
-    } else {
-        /* We have a list */
-        buf_count = 0;
-        n = PySequence_Length(data);
-        bufs = (uv_buf_t *) PyMem_Malloc(sizeof(uv_buf_t) * n);
-        if (!bufs) {
-            PyErr_NoMemory();
-            goto error;
-        }
-        for (i = 0;i < n; i++) {
-            item = PySequence_GetItem(data, i);
-            if (!item || !PyString_Check(item))
-                continue;
-            data_len = PyString_Size(item);
-            data_str = PyString_AsString(item);
-            tmp = (char *) PyMem_Malloc(data_len + 1);
-            if (!tmp)
-                continue;
-            strcpy(tmp, data_str);
-            tmpbuf = uv_buf_init(tmp, data_len);
-            bufs[i] = tmpbuf;
-            buf_count++;
-        }
-
+    bufs = (uv_buf_t *) PyMem_Malloc(sizeof(uv_buf_t));
+    if (!bufs) {
+        PyErr_NoMemory();
+        goto error;
     }
 
+    data_str = pbuf.buf;
+    data_len = pbuf.len;
+    tmp = (char *) PyMem_Malloc(data_len);
+    if (!tmp) {
+        PyErr_NoMemory();
+        goto error;
+    }
+    memcpy(tmp, data_str, data_len);
+    tmpbuf = uv_buf_init(tmp, data_len);
+    bufs[0] = tmpbuf;
+    buf_count = 1;
+
     write_data->bufs = bufs;
     write_data->buf_count = buf_count;
     req_data->data = (void *)write_data;
@@ -482,9 +460,12 @@ Pipe_func_write2(Pipe *self, PyObject *args)
         goto error;
     }
 
+    PyBuffer_Release(&pbuf);
+
     Py_RETURN_NONE;
 
 error:
+    PyBuffer_Release(&pbuf);
     if (bufs) {
         for (i = 0; i < buf_count; i++) {
             PyMem_Free(bufs[i].base);
