@@ -26,50 +26,70 @@
 
 #define LIBUV_VERSION UV_VERSION_MAJOR.UV_VERSION_MINOR-LIBUV_REVISION
 
+#ifdef PYUV_PYTHON3
+static PyModuleDef pyuv_module = {
+    PyModuleDef_HEAD_INIT,
+    "pyuv",                 /*m_name*/
+    NULL,                   /*m_doc*/
+    -1,                     /*m_size*/
+    NULL,                   /*m_methods*/
+};
+#endif
 
 /* Module */
-static PyObject*
-initialize_module(void)
+PyObject*
+init_pyuv(void)
 {
+    /* Modules */
+    PyObject *pyuv;
+    PyObject *errno_module;
+    PyObject *error_module;
+    PyObject *dns;
+    PyObject *fs;
+    PyObject *util;
+
     /* Initialize GIL */
     PyEval_InitThreads();
 
     /* Main module */
-    PyObject *pyuv = Py_InitModule("pyuv", NULL);
+#ifdef PYUV_PYTHON3
+    pyuv = PyModule_Create(&pyuv_module);
+#else
+    pyuv = Py_InitModule("pyuv", NULL);
+#endif
 
     /* Errno module */
-    PyObject *errno_module = init_errno();
+    errno_module = init_errno();
     if (errno_module == NULL) {
-        return NULL;
+        goto fail;
     }
     PyUVModule_AddObject(pyuv, "errno", errno_module);
 
     /* Error module */
-    PyObject *error = init_error();
-    if (error == NULL) {
-        return NULL;
+    error_module = init_error();
+    if (error_module == NULL) {
+        goto fail;
     }
-    PyUVModule_AddObject(pyuv, "error", error);
+    PyUVModule_AddObject(pyuv, "error", error_module);
 
     /* DNS module */
-    PyObject *dns = init_dns();
+    dns = init_dns();
     if (dns == NULL) {
-        return NULL;
+        goto fail;
     }
     PyUVModule_AddObject(pyuv, "dns", dns);
-    PyUVModule_AddType(dns, "DNSResolver", &DNSResolverType);
 
     /* FS module */
-    PyObject *fs = init_fs();
+    fs = init_fs();
     if (fs == NULL) {
-        return NULL;
+        goto fail;
     }
     PyUVModule_AddObject(pyuv, "fs", fs);
 
     /* Util module */
-    PyObject *util = init_util();
+    util = init_util();
     if (util == NULL) {
-        return NULL;
+        goto fail;
     }
     PyUVModule_AddObject(pyuv, "util", util);
 
@@ -119,12 +139,27 @@ initialize_module(void)
     PyModule_AddStringConstant(pyuv, "LIBUV_VERSION", __MSTR(LIBUV_VERSION));
 
     return pyuv;
+
+fail:
+#ifdef PYUV_PYTHON3
+    Py_DECREF(pyuv);
+#endif
+    return NULL;
+
 }
 
-void
+#ifdef PYUV_PYTHON3
+PyMODINIT_FUNC
+PyInit_pyuv(void)
+{
+    return init_pyuv();
+}
+#else
+PyMODINIT_FUNC
 initpyuv(void)
 {
-    initialize_module();
+    init_pyuv();
 }
+#endif
 
 
