@@ -12,12 +12,14 @@ static void
 on_signal_close(uv_handle_t *handle)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
-    ASSERT(handle);
-    Signal *self = (Signal *)handle->data;
-    ASSERT(self);
-
+    Signal *self;
     PyObject *result;
 
+    ASSERT(handle);
+
+    self = (Signal *)handle->data;
+    ASSERT(self);
+
     if (self->on_close_cb != Py_None) {
         result = PyObject_CallFunctionObjArgs(self->on_close_cb, self, NULL);
         if (result == NULL) {
@@ -51,15 +53,17 @@ static void
 on_signal_callback(uv_prepare_t *handle, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
+    Signal *self;
+
     ASSERT(handle);
     ASSERT(status == 0);
 
-    Signal *self = (Signal *)handle->data;
+    self = (Signal *)handle->data;
     ASSERT(self);
     /* Object could go out of scope in the callback, increase refcount to avoid it */
     Py_INCREF(self);
 
-    if (PyErr_CheckSignals() < 0) {
+    if (PyErr_CheckSignals() < 0 && PyErr_Occurred()) {
         PyErr_Print();
     }
 
@@ -71,12 +75,14 @@ on_signal_callback(uv_prepare_t *handle, int status)
 static PyObject *
 Signal_func_start(Signal *self)
 {
+    int r;
+
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_SignalError, "Signal is closed");
         return NULL;
     }
 
-    int r = uv_prepare_start(self->uv_handle, on_signal_callback);
+    r = uv_prepare_start(self->uv_handle, on_signal_callback);
     if (r != 0) {
         raise_uv_exception(self->loop, PyExc_SignalError);
         return NULL;
@@ -89,12 +95,14 @@ Signal_func_start(Signal *self)
 static PyObject *
 Signal_func_stop(Signal *self)
 {
+    int r;
+
     if (!self->uv_handle) {
         PyErr_SetString(PyExc_SignalError, "Signal is already closed");
         return NULL;
     }
 
-    int r = uv_prepare_stop(self->uv_handle);
+    r = uv_prepare_stop(self->uv_handle);
     if (r != 0) {
         raise_uv_exception(self->loop, PyExc_SignalError);
         return NULL;
@@ -139,6 +147,7 @@ Signal_func_close(Signal *self, PyObject *args)
 static PyObject *
 Signal_active_get(Signal *self, void *closure)
 {
+    UNUSED_ARG(closure);
     return PyBool_FromLong((long)uv_is_active((uv_handle_t *)self->uv_handle));
 }
 
@@ -146,10 +155,12 @@ Signal_active_get(Signal *self, void *closure)
 static int
 Signal_tp_init(Signal *self, PyObject *args, PyObject *kwargs)
 {
-    int r = 0;
+    int r;
+    uv_prepare_t *uv_prepare = NULL;
     Loop *loop;
     PyObject *tmp = NULL;
-    uv_prepare_t *uv_prepare = NULL;
+
+    UNUSED_ARG(kwargs);
 
     if (self->uv_handle) {
         PyErr_SetString(PyExc_SignalError, "Object already initialized");
