@@ -118,6 +118,9 @@ on_udp_read(uv_udp_t* handle, int nread, uv_buf_t buf, struct sockaddr* addr, un
             PyErr_WriteUnraisable(self->on_read_cb);
         }
         Py_XDECREF(result);
+        Py_DECREF(address_tuple);
+        Py_DECREF(data);
+        Py_DECREF(py_errorno);
     }
 
     PyMem_Free(buf.base);
@@ -162,6 +165,7 @@ on_udp_send(uv_udp_send_t* req, int status)
             PyErr_WriteUnraisable(callback);
         }
         Py_XDECREF(result);
+        Py_DECREF(py_errorno);
     }
 
     for (i = 0; i < send_data->buf_count; i++) {
@@ -292,7 +296,7 @@ UDP_func_send(UDP *self, PyObject *args)
     udp_req_data_t *req_data = NULL;
     udp_send_data_t *send_data = NULL;
     Py_ssize_t data_len;
-    PyObject *item, *data, *address_tuple, *callback;
+    PyObject *item, *data, *callback;
 
     buf_count = 0;
     callback = Py_None;
@@ -302,7 +306,7 @@ UDP_func_send(UDP *self, PyObject *args)
         return NULL;
     }
 
-    if (!PyArg_ParseTuple(args, "OO|O:send", &data, &address_tuple, &callback)) {
+    if (!PyArg_ParseTuple(args, "(si)O|O:send", &dest_ip, &dest_port, &data, &callback)) {
         return NULL;
     }
 
@@ -316,10 +320,6 @@ UDP_func_send(UDP *self, PyObject *args)
         return NULL;
     }
 
-    if (!PyArg_ParseTuple(address_tuple, "si", &dest_ip, &dest_port)) {
-        return NULL;
-    }
-
     if (dest_port < 0 || dest_port > 65536) {
         PyErr_SetString(PyExc_ValueError, "port must be between 0 and 65536");
         return NULL;
@@ -387,17 +387,22 @@ UDP_func_send(UDP *self, PyObject *args)
         }
         for (i = 0;i < n; i++) {
             item = PySequence_GetItem(data, i);
-            if (!item || !PyString_Check(item))
+            if (!item || !PyString_Check(item)) {
+                Py_XDECREF(item);
                 continue;
+            }
             data_len = PyString_Size(item);
             data_str = PyString_AsString(item);
             tmp = (char *) PyMem_Malloc(data_len + 1);
-            if (!tmp)
+            if (!tmp) {
+                Py_DECREF(item);
                 continue;
+            }
             memcpy(tmp, data_str, data_len + 1);
             tmpbuf = uv_buf_init(tmp, data_len);
             bufs[i] = tmpbuf;
             buf_count++;
+            Py_DECREF(item);
         }
 
     }
