@@ -1,11 +1,12 @@
 
-import os
+import sys
 
+from common import unittest2
 import common
 import pyuv
 
 
-class ProcessTest(common.UVTestCase):
+class ProcessTest(unittest2.TestCase):
 
     def test_process_basic(self):
         self.exit_cb_called = 0
@@ -18,7 +19,10 @@ class ProcessTest(common.UVTestCase):
             proc.close(proc_close_cb)
         loop = pyuv.Loop.default_loop()
         proc = pyuv.Process(loop)
-        proc.spawn(file="./proc_basic.py", exit_callback=proc_exit_cb)
+        if sys.platform == 'win32':
+            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_basic.py"], exit_callback=proc_exit_cb)
+        else:
+            proc.spawn(file="./proc_basic.py", exit_callback=proc_exit_cb)
         pid = proc.pid
         loop.run()
         self.assertEqual(self.exit_cb_called, 1)
@@ -36,7 +40,10 @@ class ProcessTest(common.UVTestCase):
             proc.close(proc_close_cb)
         loop = pyuv.Loop.default_loop()
         proc = pyuv.Process(loop)
-        proc.spawn(file="./proc_basic.py", exit_callback=proc_exit_cb, cwd=".")
+        if sys.platform == 'win32':
+            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_basic.py"], exit_callback=proc_exit_cb, cwd=".")
+        else:
+            proc.spawn(file="./proc_basic.py", exit_callback=proc_exit_cb, cwd=".")
         loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 1)
@@ -57,12 +64,15 @@ class ProcessTest(common.UVTestCase):
         loop = pyuv.Loop.default_loop()
         stdout_pipe = pyuv.Pipe(loop)
         proc = pyuv.Process(loop)
-        proc.spawn(file="./proc_stdout.py", exit_callback=proc_exit_cb, stdout=stdout_pipe)
+        if sys.platform == 'win32':
+            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_stdout.py"], exit_callback=proc_exit_cb, stdout=stdout_pipe)
+        else:
+            proc.spawn(file="./proc_stdout.py", exit_callback=proc_exit_cb, stdout=stdout_pipe)
         stdout_pipe.start_read(stdout_read_cb)
         loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 2)
-        self.assertEqual(self.received_output, "TEST")
+        self.assertEqual(self.received_output, b"TEST")
 
     def test_process_args(self):
         self.exit_cb_called = 0
@@ -80,12 +90,15 @@ class ProcessTest(common.UVTestCase):
         loop = pyuv.Loop.default_loop()
         stdout_pipe = pyuv.Pipe(loop)
         proc = pyuv.Process(loop)
-        proc.spawn(file="./proc_args_stdout.py", args=["TEST"], exit_callback=proc_exit_cb, stdout=stdout_pipe)
+        if sys.platform == 'win32':
+            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_args_stdout.py", b"TEST"], exit_callback=proc_exit_cb, stdout=stdout_pipe)
+        else:
+            proc.spawn(file="./proc_args_stdout.py", args=[b"TEST"], exit_callback=proc_exit_cb, stdout=stdout_pipe)
         stdout_pipe.start_read(stdout_read_cb)
         loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 2)
-        self.assertEqual(self.received_output, "TEST")
+        self.assertEqual(self.received_output, b"TEST")
 
     def test_process_env(self):
         self.exit_cb_called = 0
@@ -103,14 +116,20 @@ class ProcessTest(common.UVTestCase):
         loop = pyuv.Loop.default_loop()
         stdout_pipe = pyuv.Pipe(loop)
         proc = pyuv.Process(loop)
-        proc.spawn(file="./proc_env_stdout.py", env={"TEST": "TEST"}, exit_callback=proc_exit_cb, stdout=stdout_pipe)
+        if sys.platform == 'win32':
+            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_env_stdout.py"], env={b"TEST": b"TEST"}, exit_callback=proc_exit_cb, stdout=stdout_pipe)
+        else:
+            proc.spawn(file="./proc_env_stdout.py", env={b"TEST": b"TEST"}, exit_callback=proc_exit_cb, stdout=stdout_pipe)
         stdout_pipe.start_read(stdout_read_cb)
         loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 2)
-        self.assertEqual(self.received_output, "TEST")
+        self.assertEqual(self.received_output, b"TEST")
 
     def test_process_stdin(self):
+        if sys.platform == 'win32':
+            # TODO: fix
+            return
         self.exit_cb_called = 0
         self.close_cb_called = 0
         self.received_output = None
@@ -133,13 +152,16 @@ class ProcessTest(common.UVTestCase):
         stdin_pipe = pyuv.Pipe(loop)
         stdout_pipe = pyuv.Pipe(loop)
         proc = pyuv.Process(loop)
-        proc.spawn(file="./proc_stdin_stdout.py", exit_callback=proc_exit_cb, stdin=stdin_pipe, stdout=stdout_pipe)
+        if sys.platform == 'win32':
+            proc.spawn(file="cmd.exe", args=["/c", "proc_stdin_stdout.py"], exit_callback=proc_exit_cb, stdin=stdin_pipe, stdout=stdout_pipe)
+        else:
+            proc.spawn(file="./proc_stdin_stdout.py", exit_callback=proc_exit_cb, stdin=stdin_pipe, stdout=stdout_pipe)
         stdout_pipe.start_read(stdout_read_cb)
-        stdin_pipe.write("TEST"+os.linesep, stdin_write_cb)
+        stdin_pipe.write(b"TEST"+common.linesep, stdin_write_cb)
         loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 3)
-        self.assertEqual(self.received_output, "TEST")
+        self.assertEqual(self.received_output, b"TEST")
 
     def test_process_kill(self):
         self.exit_cb_called = 0
@@ -160,16 +182,20 @@ class ProcessTest(common.UVTestCase):
         timer = pyuv.Timer(loop)
         timer.start(timer_cb, 0.1, 0)
         proc = pyuv.Process(loop)
-        proc.spawn(file="./proc_infinite.py", exit_callback=proc_exit_cb)
+        if sys.platform == 'win32':
+            proc.spawn(file="cmd.exe", args=[b"/c", b"proc_infinite.py"], exit_callback=proc_exit_cb)
+        else:
+            proc.spawn(file="./proc_infinite.py", exit_callback=proc_exit_cb)
         loop.run()
         self.assertEqual(self.exit_cb_called, 1)
         self.assertEqual(self.close_cb_called, 2)
-        self.assertEqual(self.exit_status, 0)
+        if sys.platform == 'win32':
+            self.assertEqual(self.exit_status, 1)
+        else:
+            self.assertEqual(self.exit_status, 0)
         self.assertEqual(self.term_signal, 15)
 
 
 if __name__ == '__main__':
-    import unittest
-    tests = unittest.TestSuite(common.suites)
-    unittest.TextTestRunner(verbosity=2).run(tests)
+    unittest2.main(verbosity=2)
 
