@@ -5,6 +5,12 @@ from common import unittest2
 import common
 import pyuv
 
+try:
+    memoryview
+except NameError:
+    # Fix for Python 2.6
+    memoryview = str
+
 
 TEST_PORT = 1234
 
@@ -50,8 +56,7 @@ class TCPTest(unittest2.TestCase):
 
     def on_client_read(self, client, data, error):
         self.assertNotEqual(data, None)
-        data = data.strip()
-        self.assertEquals(data, b"PING")
+        self.assertEquals(data, b"PING"+common.linesep)
         client.close()
 
     def test_tcp1(self):
@@ -63,6 +68,93 @@ class TCPTest(unittest2.TestCase):
         self.loop.run()
 
 
+class TCPTestUnicode(unittest2.TestCase):
+
+    def setUp(self):
+        self.loop = pyuv.Loop.default_loop()
+        self.server = None
+        self.client = None
+        self.client_connections = []
+
+    def on_connection(self, server, error):
+        self.assertEqual(error, None)
+        client = pyuv.TCP(pyuv.Loop.default_loop())
+        server.accept(client)
+        self.client_connections.append(client)
+        client.start_read(self.on_client_connection_read)
+        if sys.version_info >= (3, 0):
+            data = 'PING'
+        else:
+            data = unicode('PING')
+        client.write(data)
+
+    def on_client_connection_read(self, client, data, error):
+        if data is None:
+            client.close()
+            self.client_connections.remove(client)
+            self.server.close()
+            return
+
+    def on_client_connection(self, client, error):
+        self.assertEqual(error, None)
+        client.start_read(self.on_client_read)
+
+    def on_client_read(self, client, data, error):
+        self.assertNotEqual(data, None)
+        self.assertEqual(data, b"PING")
+        client.close()
+
+    def test_tcp_unicode(self):
+        self.server = pyuv.TCP(self.loop)
+        self.server.bind(("0.0.0.0", TEST_PORT))
+        self.server.listen(self.on_connection)
+        self.client = pyuv.TCP(self.loop)
+        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
+        self.loop.run()
+
+
+class TCPTestMemoryview(unittest2.TestCase):
+
+    def setUp(self):
+        self.loop = pyuv.Loop.default_loop()
+        self.server = None
+        self.client = None
+        self.client_connections = []
+
+    def on_connection(self, server, error):
+        self.assertEqual(error, None)
+        client = pyuv.TCP(pyuv.Loop.default_loop())
+        server.accept(client)
+        self.client_connections.append(client)
+        client.start_read(self.on_client_connection_read)
+        data = memoryview(b"PING")
+        client.write(data)
+
+    def on_client_connection_read(self, client, data, error):
+        if data is None:
+            client.close()
+            self.client_connections.remove(client)
+            self.server.close()
+            return
+
+    def on_client_connection(self, client, error):
+        self.assertEqual(error, None)
+        client.start_read(self.on_client_read)
+
+    def on_client_read(self, client, data, error):
+        self.assertNotEqual(data, None)
+        self.assertEqual(data, b"PING")
+        client.close()
+
+    def test_tcp_memoryview(self):
+        self.server = pyuv.TCP(self.loop)
+        self.server.bind(("0.0.0.0", TEST_PORT))
+        self.server.listen(self.on_connection)
+        self.client = pyuv.TCP(self.loop)
+        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
+        self.loop.run()
+
+
 class TCPTestNull(unittest2.TestCase):
 
     def setUp(self):
@@ -92,8 +184,7 @@ class TCPTestNull(unittest2.TestCase):
 
     def on_client_read(self, client, data, error):
         self.assertNotEqual(data, None)
-        data = data.strip()
-        self.assertEquals(data, b"PIN\x00G")
+        self.assertEquals(data, b"PIN\x00G"+common.linesep)
         client.close()
 
     def test_tcp_null(self):
@@ -118,7 +209,7 @@ class TCPTestList(unittest2.TestCase):
         server.accept(client)
         self.client_connections.append(client)
         client.start_read(self.on_client_connection_read)
-        client.write([b"PING1", b"PING2", b"PING3", b"PING4", b"PING5", b"PING6", common.linesep])
+        client.writelines([b"PING1", b"PING2", b"PING3", b"PING4", b"PING5", b"PING6", b"PING7", b"PING8", b"PING9", b"PING10", b"PING11", b"PING12"])
 
     def on_client_connection_read(self, client, data, error):
         if data is None:
@@ -133,8 +224,7 @@ class TCPTestList(unittest2.TestCase):
 
     def on_client_read(self, client, data, error):
         self.assertNotEqual(data, None)
-        data = data.strip()
-        self.assertEquals(data, b"PING1PING2PING3PING4PING5PING6")
+        self.assertEquals(data, b"PING1PING2PING3PING4PING5PING6PING7PING8PING9PING10PING11PING12")
         client.close()
 
     def test_tcp_list(self):
@@ -146,6 +236,49 @@ class TCPTestList(unittest2.TestCase):
         self.loop.run()
 
 
+class TCPTestListUnicode(unittest2.TestCase):
+
+    def setUp(self):
+        self.loop = pyuv.Loop.default_loop()
+        self.server = None
+        self.client = None
+        self.client_connections = []
+
+    def on_connection(self, server, error):
+        client = pyuv.TCP(pyuv.Loop.default_loop())
+        server.accept(client)
+        self.client_connections.append(client)
+        client.start_read(self.on_client_connection_read)
+        if sys.version_info >= (3, 0):
+            data = 'PING'
+        else:
+            data = unicode('PING')
+        client.writelines([data for x in range(100)])
+
+    def on_client_connection_read(self, client, data, error):
+        if data is None:
+            client.close()
+            self.client_connections.remove(client)
+            self.server.close()
+            return
+
+    def on_client_connection(self, client, error):
+        self.assertEquals(error, None)
+        client.start_read(self.on_client_read)
+
+    def on_client_read(self, client, data, error):
+        self.assertEquals(data, b"PING"*100)
+        client.close()
+
+    def test_tcp_list_unicode(self):
+        self.server = pyuv.TCP(self.loop)
+        self.server.bind(("0.0.0.0", TEST_PORT))
+        self.server.listen(self.on_connection)
+        self.client = pyuv.TCP(self.loop)
+        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
+        self.loop.run()
+
+
 class TCPTestListNull(unittest2.TestCase):
 
     def setUp(self):
@@ -159,7 +292,7 @@ class TCPTestListNull(unittest2.TestCase):
         server.accept(client)
         self.client_connections.append(client)
         client.start_read(self.on_client_connection_read)
-        client.write([b"PING1", b"PING2", b"PING3", b"PING4", b"PING5", b"PING\x00\xFF6", common.linesep])
+        client.writelines([b"PING1", b"PING2", b"PING3", b"PING4", b"PING5", b"PING\x00\xFF6"])
 
     def on_client_connection_read(self, client, data, error):
         if data is None:
@@ -174,7 +307,6 @@ class TCPTestListNull(unittest2.TestCase):
 
     def on_client_read(self, client, data, error):
         self.assertNotEqual(data, None)
-        data = data.strip()
         self.assertEquals(data, b"PING1PING2PING3PING4PING5PING\x00\xFF6")
         client.close()
 
@@ -201,13 +333,10 @@ class TCPTestInvalidData(unittest2.TestCase):
         self.client_connections.append(client)
         client.start_read(self.on_client_connection_read)
 
-        if sys.version_info >= (3, 0):
-            data = 'Unicode'
-        else:
-            data = unicode('Unicode')
-
-        self.assertRaises(TypeError, client.write, data)
-        self.assertRaises(TypeError, client.write, 1)
+        self.assertRaises(TypeError, client.write, 1234)
+        self.assertRaises(TypeError, client.write, object())
+        self.assertRaises(TypeError, client.writelines, 1234)
+        self.assertRaises(TypeError, client.writelines, object())
 
         client.close()
         self.client_connections.remove(client)
@@ -272,8 +401,7 @@ class TCPShutdownTest(unittest2.TestCase):
 
     def on_client_read(self, client, data, error):
         self.assertNotEqual(data, None)
-        data = data.strip()
-        self.assertEquals(data, b"PING")
+        self.assertEquals(data, b"PING"+common.linesep)
         client.shutdown(self.on_client_shutdown)
 
     def test_tcp_shutdown(self):
